@@ -373,6 +373,7 @@
 #define DEFAULT_HIGH_BIT_ESCAPE 2
 #define DEFAULT_HIGHLIGHT_CHAR "BOLD"
 #define DEFAULT_HISTORY 150
+#define DEFAULT_HISTORY_CIRCLEQ 1
 #define DEFAULT_HOLD_INTERVAL 10
 #define DEFAULT_HOLD_MODE 0
 #define DEFAULT_INDENT 0
@@ -484,6 +485,7 @@
 #define DEFAULT_STATUS_WINDOW "^^^^^^^^"
 #define DEFAULT_SUPPRESS_FROM_REMOTE_SERVER 0
 #define DEFAULT_SUPPRESS_SERVER_MOTD 0
+#define DEFAULT_SWITCH_CHANNEL_ON_PART 1
 #define DEFAULT_TAB 1
 #define	DEFAULT_TAB_MAX 0
 #define DEFAULT_TERM_DOES_BRIGHT_BLINK 0
