@@ -1434,9 +1434,13 @@ FILE *	uzfopen (char **filename, char *path, int do_error)
 	 */
 	if (**filename == '~')
 	{
-		filename_blah = expand_twiddle(*filename);
-		filename_expanded = LOCAL_COPY(filename_blah);
-		new_free(&filename_blah);
+		if ((filename_blah = expand_twiddle(*filename)))
+		{
+			filename_expanded = LOCAL_COPY(filename_blah);
+			new_free(&filename_blah);
+		}
+		else
+			filename_expanded = LOCAL_COPY(*filename);
 	}
 	else
 		filename_expanded = LOCAL_COPY(*filename);
