@@ -345,7 +345,7 @@ void 	add_channel (const char *name, int server)
 		new_c->server = server;
 	}
 	else if (was_current)
-		panic("Channel [%s] is current but it couldn't find it!",
+		panic("Channel [%s] is current but I couldn't find it!",
 			name);
 	else
 		new_c = create_channel(name, server);
@@ -1388,7 +1388,8 @@ const char *	fetch_userhost (int server, const char *nick)
 
 	while (traverse_all_channels(&tmp, server))
 	{
-		if ((user = find_nick_on_channel(tmp, nick)))
+		if ((user = find_nick_on_channel(tmp, nick)) && 
+				user->userhost)
 			return user->userhost;
 	}
 
@@ -1434,14 +1435,13 @@ void 	set_channel_window (Window *window, const char *channel)
 
 	if (!(tmp = find_channel(channel, window->server)))
 	{
-		if (window->server != tmp->server)
-			yell("WARNING! set_channel_window is moving a channel "
-				"from a window on one server to a window on "
-				"another server.  This is probably a bug. "
-				"[%s] [%d] [%d]", channel, tmp->server,
-							window->server);
+		yell("WARNING! set_channel_window is acting on a channel "
+			"that doesn't exist even though the window is "
+			"expecting it.  This is probably a bug. "
+			"[%s] [%d]", channel, window->server);
 	}
-	tmp->window = window;
+	else
+		tmp->window = window;
 }
 
 /*
@@ -1537,13 +1537,12 @@ char *	create_channel_list (int server)
 
 void 	channel_server_delete (int i)
 {
-	Channel	*tmp = NULL;
+	Channel	*tmp;
 
-	if (i >= 0)
+	for (tmp = channel_list; tmp; tmp = tmp->next)
 	{
-		while (traverse_all_channels(&tmp, i))
-			if (tmp->server >= i)
-				tmp->server--;
+		if (tmp->server >= i)
+			tmp->server--;
 	}
 }
 
