@@ -1,5 +1,5 @@
 /*
- * numbers.c:handles all those strange numeric response dished out by that
+ * numbers.c: handles all those strange numeric response dished out by that
  * wacky, nutty program we call ircd 
  *
  * written by michael sandrof
@@ -21,368 +21,111 @@ static	char	rcsid[] = "$Id: numbers.c,v 1.12 1993/12/30 12:45:35 mrgreen Exp $";
 #include "lastlog.h"
 #include "hook.h"
 #include "server.h"
-#include "whois.h"
 #include "numbers.h"
 #include "window.h"
 #include "screen.h"
 #include "output.h"
 #include "names.h"
-#include "whois.h"
 #include "funny.h"
 #include "parse.h"
-#include "edit.h"
+#include "commands.h"
 #include "notice.h"
+#include "notify.h"
+#include "vars.h"
+#include "who.h"
+#include "alias.h"
 
-static void fudge_nickname _((void));
-
-static	int	already_doing_reset_nickname = 0;
 static 	int	number_of_bans = 0;
-extern	int	in_e_nick;
 
 /*
  * numeric_banner: This returns in a static string of either "xxx" where
  * xxx is the current numeric, or "***" if SHOW_NUMBERS is OFF 
  */
-extern char	*
-numeric_banner _((void))
+char	*numeric_banner (void)
 {
-	static	char	thing[4];
+	static	char	thing[80];
+	char *str;
 
-	if (get_int_var(SHOW_NUMERICS_VAR))
+	if (current_numeric < 0 && get_int_var(SHOW_NUMERICS_VAR))
 		sprintf(thing, "%3.3u", -current_numeric);
+	else if ((str = get_string_var(BANNER_VAR)))
+	{
+		if (get_int_var(BANNER_EXPAND_VAR))
+		{
+			int af;
+			char *foo = expand_alias(str, empty_string, &af, NULL);
+			strmcpy(thing, foo, 79);
+			new_free(&foo);
+		}
+		else
+			strlcpy(thing, str, 80);
+	}
 	else
-		strcpy(thing, "***");
+		*thing = 0;
+
 	return (thing);
 }
 
 
 /*
  * display_msg: handles the displaying of messages from the variety of
- * possible formats that the irc server spits out.  you'd think someone would
- * simplify this 
+ * possible formats that the irc server spits out.
+ *
+ * Simplified by Jeremy Nelson (esl) some time in 1996.
+ * -- called by more than one place.
  */
-#ifdef __STDC__
-extern void display_msg (char *from, char **ArgList)
-#else
-extern void
-display_msg(from, ArgList)
-	char	*from,
-		**ArgList;
-#endif
+void 	display_msg (char *from, char **ArgList)
 {
-	char	*ptr;
+	char	*ptr = NULL;
 	char	*rest;
+	int	drem;
 
 	rest = PasteArgs(ArgList, 0);
 	if (from && (my_strnicmp(get_server_itsname(from_server), from,
 			strlen(get_server_itsname(from_server))) == 0))
-		from = (char *) 0;
-	if ((ptr = (char *) index(rest, ':')) != NULL)
-	{
-		*(ptr++) = (char) 0;
-		if (strlen(rest))
-		{
-			if ((from) && (!get_int_var(SUPPRESS_FROM_REMOTE_SERVER)))
-				put_it("%s %s: %s (from %s)", numeric_banner(),
-					rest, ptr, from);
-			else
-				put_it("%s %s: %s", numeric_banner(), rest,
-					ptr);
-		}
-		else
-		{
-			if ((from) && (!get_int_var(SUPPRESS_FROM_REMOTE_SERVER)))
-				put_it("%s %s (from %s)", numeric_banner(),
-					ptr, from);
-			else
-				put_it("%s %s", numeric_banner(), ptr);
-		}
-	}
+		from = NULL;
+
+	/* This fix by SrfRog, again by |Rain| */
+	ptr = strchr(rest, ':');
+	if (ptr && ptr > rest && ptr[-1] == ' ')	/* per RFC 1459 */
+		*ptr++ = 0;
 	else
-	{
-		if ((from) && (!get_int_var(SUPPRESS_FROM_REMOTE_SERVER)))
-			put_it("%s %s (from %s)", numeric_banner(), rest, from);
-		else
-			put_it("%s %s", numeric_banner(), rest);
-	}
+		ptr = NULL;
+
+        drem = (from) && (!get_int_var(SUPPRESS_FROM_REMOTE_SERVER));
+
+        /*
+         * This handles all the different cases of server messages.
+         * If you dont believe me, try it out. :P
+         *
+         * There are 16 distinct possibilities, since there are 4
+         * independant variables.  In practice only about 6 to 8 of 
+	 * the possibilities are used.
+         */
+        put_it("%s %s%s%s%s%s%s%s",
+                numeric_banner(),
+                strlen(rest)        ? rest     : empty_string,
+                strlen(rest) && ptr ? ":"      : empty_string,
+                strlen(rest)        ? space    : empty_string,
+                ptr                 ? ptr      : empty_string,
+                drem                ? "(from " : empty_string,
+                drem                ? from     : empty_string,
+                drem                ? ")"      : empty_string
+              );
 }
 
-/*
- * password_sendline: called by send_line() in get_password() to handle
- * hitting of the return key, etc 
- */
-#ifdef __STDC__
-static void password_sendline (char *data, char *line)
-#else
-static	void
-password_sendline(data, line)
-	char	*data;
-	char	*line;
-#endif
-{
-	int	new_server;
-
-	new_server = atoi(line);
-	set_server_password(new_server, line);
-	connect_to_server_by_refnum(new_server, -1);
-}
-
-/*
- * get_password: when a host responds that the user needs to supply a
- * password, it gets handled here!  the user is prompted for a password and
- * then reconnection is attempted with that password.  but, the reality of
- * the situation is that no one really uses user passwords.  ah well 
- */
-static	void
-get_password _((void))
-{
-	char	server_num[8];
-
-	say("password required for connection to server %s",
-		get_server_name(from_server));
-	close_server(from_server, empty_string);
-        if (!dumb)
-	{
-		strcpy(server_num, ltoa(from_server));
-		add_wait_prompt("Server Password:", password_sendline,
-                       server_num, WAIT_PROMPT_LINE);
-	}
-}
-
-#ifdef __STDC__
-extern void nickname_sendline (char *data, char *nick)
-#else
-extern	void
-nickname_sendline(data, nick)
-	char	*data;
-	char	*nick;
-#endif
-{
-	int	new_server, server;
-
-	new_server = atoi(data);
-	if ((nick = check_nickname(nick)) != NULL)
-	{
-		server = from_server;
-		from_server = new_server;
-		send_to_server("NICK %s", nick);
-		if (new_server == primary_server)
-			strmcpy(nickname, nick, 
-#ifdef ALLOW_LONG_NICKNAMES
-						LONG_NICKNAME_LEN
-#else
-						NICKNAME_LEN
-#endif
-								  );
-		set_server_nickname(new_server, nick);
-		from_server = server;
-		already_doing_reset_nickname = 0;
-		update_all_status();
-	}
-	else
-	{
-		say("illegal nickname, try again");
-		if (!dumb)
-			add_wait_prompt("Nickname: ", nickname_sendline, data,
-					WAIT_PROMPT_LINE);
-	}
-}
-
-/*
- * reset_nickname: when the server reports that the selected nickname is not
- * a good one, it gets reset here. 
- */
-static void reset_nickname _((void))
-{
-	char	server_num[10];
-
-	if (already_doing_reset_nickname)
-		return;
-	say("You have specified an illegal nickname");
-	if (!dumb)
-	{
-		already_doing_reset_nickname = 1;
-		say("Please enter your nickname");
-		strcpy(server_num, ltoa(from_server));
-		add_wait_prompt("Nickname: ", nickname_sendline, server_num,
-			WAIT_PROMPT_LINE);
-	}
-	update_all_status();
-}
-
-#ifdef __STDC__
-static void channel_topic (char *from, char **ArgList)
-#else
-static	void
-channel_topic(from, ArgList)
-	char	*from,
-		**ArgList;
-#endif
-{
-	message_from(ArgList[0], LOG_CRAP);
-	put_it("%s Topic for %s: %s", numeric_banner(), ArgList[0], ArgList[1]);
-}
-
-#ifdef __STDC__
-static void nickname_in_use (char *from, char **ArgList)
-#else
-static void
-nickname_in_use(from, ArgList)
-	char	*from,
-		**ArgList;
-#endif
-{
-	PasteArgs(ArgList, 0);
-	fudge_nickname();
-	if (!never_connected)
-		if (do_hook(current_numeric, "%s", *ArgList))
-			display_msg(from, ArgList);
-}
-
-#ifdef __STDC__
-static void not_valid_channel (char *from, char **ArgList)
-#else
-static	void
-not_valid_channel(from, ArgList)
-	char	*from,
-		**ArgList;
-#endif
-{
-	char	*channel;
-	char	*s;
-
-	if (!(channel = ArgList[0]) || !ArgList[1])
-		return;
-	PasteArgs(ArgList, 1);
-	s = get_server_name(from_server);
-	if (0 == my_strnicmp(s, from, strlen(s)))
-	{
-/* hack cause channel may be null */
-		if (channel)
-		{
-			remove_channel(channel, from_server);
-			put_it("%s %s %s", numeric_banner(), channel, ArgList[1]);
-		}
-		else
-			put_it("%s You did not specify a channel", numeric_banner());
-	}
-}
-
-/* from ircd .../include/numeric.h */
-/*
-#define ERR_TOOMANYCHANN     405
-#define ERR_CHANNELISFULL    471
-#define ERR_INVITEONLYCHAN   473
-#define ERR_BANNEDFROMCHAN   474
-#define ERR_BADCHANNELKEY    475
-#define ERR_BADCHANMASK      476
-*/
-#ifdef __STDC__
-static void cannot_join_channel (char *from, char **ArgList)
-#else
-static	void
-cannot_join_channel(from, ArgList)
-	char	*from,	
-		**ArgList;
-#endif
-{
-	char buffer[BIG_BUFFER_SIZE + 1];
-
-	if (ArgList[0])
-		remove_channel(ArgList[0], from_server);
-	PasteArgs(ArgList, 0);
-	strcpy(buffer, ArgList[0]);
-	switch(-current_numeric)
-	{
-	case 471:
-		strcat(buffer, " (Channel is full)");
-		break;
-	case 473:
-		strcat(buffer, " (You must be invited)");
-		break;
-	case 474:
-		strcat(buffer, " (You are banned)");
-		break;
-	case 475:
-		strcat(buffer, " (You must give the correct key)");
-		break;
-	case 476:
-		strcat(buffer, " (Bad channel mask)");
-		break;
-	}
-	put_it("%s %s", numeric_banner(), buffer);
-}
-
-
-#ifdef __STDC__
-static void version (char *from, char **ArgList)
-#else
-static	void
-version(from, ArgList)
-	char	*from,
-		**ArgList;
-#endif
-{
-#ifdef COMPAT_27
-	if (ArgList[2])
-	{
-#endif
-		PasteArgs(ArgList, 2);
-		put_it("%s Server %s: %s %s", numeric_banner(), ArgList[1],
-			ArgList[0], ArgList[2]);
-#ifdef COMPAT_27
-	}
-	else
-	{
-		PasteArgs(ArgList, 1);
-		put_it("%s Server %s: %s", numeric_banner(), ArgList[1],
-			ArgList[0]);
-	}
-#endif
-}
-
-
-#ifdef __STDC__
-static void invite (char *from, char **ArgList)
-#else
-static	void
-invite(from, ArgList)
-	char	*from,
-		**ArgList;
-#endif
-{
-	char	*who,
-		*channel;
-
-	if ((who = ArgList[0]) && (channel = ArgList[1]))
-	{
-		message_from(channel, LOG_CRAP);
-		if (do_hook(current_numeric, "%s %s %s", from, who, channel))
-			put_it("%s Inviting %s to channel %s",
-					numeric_banner(), who, channel);
-	}
-}
-
-
 /*
  * numbered_command: does (hopefully) the right thing with the numbered
  * responses from the server.  I wasn't real careful to be sure I got them
  * all, but the default case should handle any I missed (sorry) 
  */
-#ifdef __STDC__
-extern void numbered_command (char *from, int comm, char **ArgList)
-#else
-extern void
-numbered_command(from, comm, ArgList)
-	char	*from;
-	int	comm;
-	char	**ArgList;
-#endif
+void 	numbered_command (char *from, int comm, char **ArgList)
 {
 	char	*user;
 	char	none_of_these = 0;
 	int	flag,
 		lastlog_level;
+	int	old_current_numeric = current_numeric;
 
 	if (!ArgList[0] || !ArgList[1])
 		return;
@@ -404,12 +147,17 @@ numbered_command(from, comm, ArgList)
 	 * we just kind of take it at its word.
 	 */
 	case 001:	/* #define RPL_WELCOME          001 */
-		set_server2_8(from_server, 1);
-		set_server_nickname(from_server, user);
+	{
+		accept_server_nickname(from_server, user);
+		set_server_motd(from_server, 1);
+		server_is_connected(from_server, 1);
+
+		userhostbase(NULL, got_my_userhost, 1);
 		PasteArgs(ArgList, 0);
 		if (do_hook(current_numeric, "%s %s", from, *ArgList)) 
 			display_msg(from, ArgList);
 		break;
+	}
 
 	/* 
 	 * Now instead of the terribly horrible hack using numeric 002
@@ -417,15 +165,26 @@ numbered_command(from, comm, ArgList)
 	 * numeric which is what is the most logical choice for it.
 	 */
 	case 004:	/* #define RPL_MYINFO           004 */
+	{
 		got_initial_version_28(ArgList);
 		PasteArgs(ArgList, 0);
 		if (do_hook(current_numeric, "%s %s", from, *ArgList))
 			display_msg(from, ArgList);
 		break;
+	}
+
+	case 14:		/* Erf/TS4 "cookie" numeric	014 */
+		set_server_cookie(from_server, ArgList[0]);
+		break;
 
 	case 301:		/* #define RPL_AWAY             301 */
-		user_is_away(from, ArgList);
-		break;
+        {
+                PasteArgs(ArgList, 1);
+                if (do_hook(current_numeric, "%s %s", ArgList[0], ArgList[1]))
+                        put_it("%s %s is away: %s", numeric_banner(),
+                                ArgList[0], ArgList[1]);
+                break;
+	}
 
 	case 302:		/* #define RPL_USERHOST         302 */
 		userhost_returned(from, ArgList);
@@ -436,52 +195,128 @@ numbered_command(from, comm, ArgList)
 		break;
 
 	case 311:		/* #define RPL_WHOISUSER        311 */
-		whois_name(from, ArgList);
-		break;
+        {
+                PasteArgs(ArgList, 4);
+                message_from(NULL, LOG_CRAP);
+                if (do_hook(current_numeric, "%s %s %s %s %s %s",
+                                from, ArgList[0], ArgList[1], ArgList[2],
+                                ArgList[3], ArgList[4]))
+                        put_it("%s %s is %s@%s (%s)", numeric_banner(),
+                                ArgList[0], ArgList[1], ArgList[2], ArgList[4]);
+                break;
+        }
 
 	case 312:		/* #define RPL_WHOISSERVER      312 */
-		whois_server(from, ArgList);
+	{
+		if (do_hook(current_numeric, "%s %s %s %s", from, ArgList[0], ArgList[1], ArgList[2]))
+			put_it("%s on irc via server %s (%s)", numeric_banner(),
+				ArgList[1], ArgList[2]);
 		break;
+	}
 
 	case 313:		/* #define RPL_WHOISOPERATOR    313 */
-		whois_oper(from, ArgList);
+	{
+		PasteArgs(ArgList, 1);
+		if (do_hook(current_numeric, "%s %s %s", from, ArgList[0], ArgList[1]))
+			put_it("%s %s %s", numeric_banner(), ArgList[0], ArgList[1]);
 		break;
+	}
 
 	case 314:		/* #define RPL_WHOWASUSER       314 */
-		whowas_name(from, ArgList);
+	{
+		PasteArgs(ArgList, 4);
+		message_from(NULL, LOG_CRAP);
+		if (do_hook(current_numeric, "%s %s %s %s %s %s",
+				from, ArgList[0], ArgList[1], ArgList[2],
+				ArgList[3], ArgList[4]))
+			put_it("%s %s was %s@%s (%s)", numeric_banner(),
+				ArgList[0], ArgList[1], ArgList[2], ArgList[4]);
 		break;
+	}
 
 	case 315:		/* #define RPL_ENDOFWHO         315 */
 	{
-		extern int doing_who;
-
-		if (cannot_open != (char *) 0)
-			yell("Cannot open: %s", cannot_open);
-
-		/* XXX oops.. forgot this. >;-) */
-		doing_who = 0;
-
-		/* Ooops. found by murple */
-		if (get_int_var(SHOW_END_OF_MSGS_VAR))
-		if (do_hook(current_numeric, "%s %s", from, ArgList[0]))
-			put_it("%s %s %s",numeric_banner(), from, ArgList[0]);
+		PasteArgs(ArgList, 0);
+		who_end(from, ArgList);
+		break;
 	}
 
-
 	case 316:		/* supported, but deprecated */
 		break;
 
 	case 317:		/* #define RPL_WHOISIDLE        317 */
-		whois_lastcom(from, ArgList);
+	{
+		char	flag, *nick, *idle_str, *startup_str;
+		int	idle;
+		time_t	startup;
+
+		nick = ArgList[0];
+		idle_str = ArgList[1];
+		startup_str = ArgList[2];
+
+		if (ArgList[3])	/* undernet */
+		{
+			PasteArgs(ArgList, 3);
+			if (nick && idle_str && do_hook(current_numeric, "%s %s %s %s %s",
+						from, nick, idle_str, startup_str, ArgList[3]))
+			{
+				if ((idle = atoi(idle_str)) > 59)
+				{
+					idle /= 60;
+					flag = 1;
+				}
+				else
+					flag = 0;
+
+				if ((startup = atol(startup_str)) != 0)
+					put_it ("%s %s has been idle %d %ss, signed on at %s",
+						numeric_banner(), nick, idle, 
+						flag?"minute":"second",my_ctime(startup));
+				else
+					put_it("%s %s has been idle %d %ss", 
+						numeric_banner(), nick, idle, 
+						flag? "minute": "second");
+			}
+		}
+		else	/* efnet */
+		{
+			PasteArgs(ArgList, 1);
+			if (nick && idle_str && 
+			    do_hook(current_numeric, "%s %s %s", from, nick, idle_str))
+			{
+				if ((idle = atoi(idle_str)) > 59)
+				{
+					idle /= 60;
+					flag = 1;
+				}
+				else
+					flag = 0;
+
+				put_it ("%s %s has been idle %d %ss",
+					numeric_banner(), nick, idle, 
+					flag?"minute":"second");
+			}
+		}
+
 		break;
+	}
 
 	case 318:		/* #define RPL_ENDOFWHOIS       318 */
-		end_of_whois(from, ArgList);
-		break;
+        {
+                PasteArgs(ArgList, 0);
+                if (do_hook(current_numeric, "%s %s", from, ArgList[0]))
+                        if (get_int_var(SHOW_END_OF_MSGS_VAR))
+                                display_msg(from, ArgList);
+                break;
+        }
 
 	case 319:		/* #define RPL_WHOISCHANNELS    319 */
-		whois_channels(from, ArgList);
+	{
+		PasteArgs(ArgList, 1);
+		if (do_hook(current_numeric, "%s %s %s", from, ArgList[0], ArgList[1]))
+			put_it("%s on channels: %s", numeric_banner(), ArgList[1]);
 		break;
+	}
 
 	case 321:		/* #define RPL_LISTSTART        321 */
 		ArgList[0] = "Channel\0Users\0Topic";
@@ -499,9 +334,28 @@ numbered_command(from, comm, ArgList)
 		funny_mode(from, ArgList);
 		break;
 
-	case 341:		/* #define RPL_INVITING         341 */
-		invite(from, ArgList);
+	case 340:		/* #define RPL_INVITING_OTHER	340 */
+	{
+		if (ArgList[2])
+		{
+			message_from(ArgList[0], LOG_CRAP);
+			if (do_hook(current_numeric, "%s %s %s %s", from, ArgList[0], ArgList[1], ArgList[2]))
+				put_it("%s %s has invited %s to channel %s", numeric_banner(), ArgList[0], ArgList[1], ArgList[2]);
+		}
 		break;
+	}
+
+	case 341:		/* #define RPL_INVITING         341 */
+	{
+		if (ArgList[1])
+		{
+			message_from(ArgList[1], LOG_CRAP);
+			if (do_hook(current_numeric, "%s %s %s", from, ArgList[0], ArgList[1]))
+				put_it("%s Inviting %s to channel %s", numeric_banner(), ArgList[0], ArgList[1]);
+		}
+		break;
+	}
+
 
 	case 352:		/* #define RPL_WHOREPLY         352 */
 		whoreply((char *) 0, ArgList);
@@ -511,21 +365,27 @@ numbered_command(from, comm, ArgList)
 		funny_namreply(from, ArgList);
 		break;
 
+	case 354:		/* #define RPL_XWHOREPLY	354 */
+		xwhoreply(NULL, ArgList);
+		break;
+
 	case 366:		/* #define RPL_ENDOFNAMES       366 */
 	{
 		int	flag = 1;
 
-		PasteArgs(ArgList, 0);
+		PasteArgs(ArgList, 1);
+		message_from(ArgList[0], LOG_CRAP);
 		if (get_int_var(SHOW_END_OF_MSGS_VAR))
-			flag = do_hook(current_numeric, "%s %s", from, ArgList[0]);
+			flag = do_hook(current_numeric, "%s %s %s", from, ArgList[0], ArgList[1]);
+		message_from(NULL, LOG_CURRENT);
 	
-		if (!funny_is_ignore_channel())
+		if (!channel_is_syncing(ArgList[0], from_server))
 		{
+			PasteArgs(ArgList, 0);
 			if (get_int_var(SHOW_END_OF_MSGS_VAR) && flag)
 				display_msg(from, ArgList);
 		}
-		else
-			funny_set_ignore_mode();
+
 		break;
 	}
 
@@ -551,8 +411,13 @@ numbered_command(from, comm, ArgList)
 	{
 		if (get_int_var(SHOW_END_OF_MSGS_VAR))
 		{
+#ifdef IRCII_LIKE_BAN_SUMMARY
+			if (do_hook(current_numeric, "%s %s %d", 
+				from, *ArgList, number_of_bans))
+#else
 			if (do_hook(current_numeric, "%s %d %s", 
 				from, number_of_bans, *ArgList))
+#endif
 			{
 				put_it("%s Total number of bans on %s - %d",
 					numeric_banner(), ArgList[0], 
@@ -562,10 +427,14 @@ numbered_command(from, comm, ArgList)
 		number_of_bans = 0;
 		break;
 	}
-	/* I put these here so that if you have SUPPRESS_SERVER_MOTD set
+
+	/* 
+	 * I put these here so that if you have SUPPRESS_SERVER_MOTD set
 	 * to on, then you cant hook 372, 375, 376, as advertised.
 	 */
 	case 372:		/* #define RPL_MOTD             372 */
+	case 377:		/* #define FORCE_RPL_MOTD	377 */
+	{
 		if (!get_int_var(SUPPRESS_SERVER_MOTD_VAR) ||
 		    !get_server_motd(from_server))
 		{
@@ -574,8 +443,9 @@ numbered_command(from, comm, ArgList)
 				put_it("%s %s", numeric_banner(), ArgList[0]);
 		}
 		break;
-
+	}
 	case 375:		/* #define RPL_MOTDSTART        375 */
+	{
 		if (!get_int_var(SUPPRESS_SERVER_MOTD_VAR) ||
 		    !get_server_motd(from_server))
 		{
@@ -584,8 +454,9 @@ numbered_command(from, comm, ArgList)
 				put_it("%s %s", numeric_banner(), ArgList[0]);
 		}
 		break;
-
+	}
 	case 376:		/* #define RPL_ENDOFMOTD        376 */
+	{
 		if (get_int_var(SHOW_END_OF_MSGS_VAR) &&
 		    (!get_int_var(SUPPRESS_SERVER_MOTD_VAR) ||
 		    !get_server_motd(from_server)))
@@ -596,52 +467,182 @@ numbered_command(from, comm, ArgList)
 		}
 		set_server_motd(from_server, 0);
 		break;
-
+	}
 	case 381: 		/* #define RPL_YOUREOPER        381 */
 		PasteArgs(ArgList, 0);
 		if (!is_server_connected(from_server))
-		{
 			say("Odd server stuff from %s: %s", from, ArgList[0]);
-			return;
-		}
-		if (do_hook(current_numeric, "%s %s", from, *ArgList))
+		else if (do_hook(current_numeric, "%s %s", from, ArgList[0]))
 			display_msg(from, ArgList);
 		break;
 
 	case 401:		/* #define ERR_NOSUCHNICK       401 */
-		no_such_nickname(from, ArgList);
-		break;
+	{
+		PasteArgs(ArgList, 1);
+		if (do_hook(current_numeric, "%s %s %s", from, ArgList[0], ArgList[1]))
+			put_it("%s %s: %s", numeric_banner(), ArgList[0], ArgList[1]);
+		notify_mark(ArgList[0], 0, 0);
 
+		if (get_int_var(AUTO_WHOWAS_VAR))
+		{
+			int foo = get_int_var(NUM_OF_WHOWAS_VAR);
+
+			if (foo > -1)
+				send_to_server("WHOWAS %s %d", ArgList[0], foo);
+			else
+				send_to_server("WHOWAS %s", ArgList[0]);
+		}
+		break;
+	}
 	case 421:		/* #define ERR_UNKNOWNCOMMAND   421 */
-		if (check_screen_redirect(ArgList[0]))
+	{
+		if (check_server_redirect(ArgList[0]))
 			break;
 		if (check_wait_command(ArgList[0]))
 			break;
+
 		PasteArgs(ArgList, 0);
-		flag = do_hook(current_numeric, "%s %s", from, *ArgList);
-		if (flag)
+		if (do_hook(current_numeric, "%s %s", from, ArgList[0]))
 			display_msg(from, ArgList);
 		break;
-
+	}
 	case 432:		/* #define ERR_ERRONEUSNICKNAME 432 */
+	{
+		if (get_int_var(AUTO_NEW_NICK_VAR))
+			fudge_nickname(from_server);
+		else
+			reset_nickname(from_server);
+
 		PasteArgs(ArgList, 0);
 		if (do_hook(current_numeric, "%s %s", from, *ArgList))
 			display_msg(from, ArgList);
-		fudge_nickname();
-		break;
 
-	case 433:		/* #define ERR_NICKNAMEINUSE    433 */ 
-		nickname_in_use(from, ArgList);
 		break;
+	}
+
+	case 437:		/* av2.9's "Nick collision" numeric 437 */
+				/* Also, undernet/dalnet "You are banned" */
+				/* Also, av2.10's "Can't do that" numeric */
+				/* Also, cs's "too many nick changes" num */
+	{
+		/*
+		 * Ugh. What a total trainwreck this is.  Sometimes, I
+		 * really hate all the ircd's out there in the world that
+		 * have to be supported.
+		 *
+		 * Well, there are at least four different, occasionally
+		 * scrutable ways we can get this numeric.
+		 *
+		 * 1) On ircnet -- As an unregistered user, the NICK that
+		 *	we are trying to register was used in the past 90
+		 *	seconds or so.  The server expects us to send
+		 *	another NICK request.
+		 *		ARGV[0] IS NICK, REGISTERED IS NO
+		 * 2) On ircnet -- As a registered user, we are trying to
+		 *	join a channel that was netsplit in the past 24 hours
+		 *	or so.  The server expects us not to do anything.
+		 *		ARGV[0] IS CHANNEL, REGISTERED IS YES
+		 * 3) On undernet/dalnet -- As a registered user, who is
+		 *	on a channel where we are banned, a NICK request
+		 *	was rejected (because we are banned).  The server
+		 *	expects us not to do anything.
+		 *		ARGV[0] IS CHANNEL, REGISTERED IS YES
+		 * 4) On a comstud efnet servers -- As a registered user, 
+		 *	we have changed our nicknames too many times in
+		 *	too short a time.  The server expects us not to do
+		 *	anything.
+		 *		ARGV[0] IS ERROR, ARGV[1] IS NULL.
+		 */
+
+		/*
+		 * Weed out the comstud one first, since it's the most bizarre.
+		 */
+		if (ArgList[0] && ArgList[1] == NULL)
+		{
+			if (do_hook(current_numeric, "%s %s", from, ArgList[0]))
+				display_msg(from, ArgList);
+			break;
+		}
+
+		/*
+		 * Now if it's a channel, it might be ircnet telling us we
+		 * can't join the channel, or undernet telling us that we 
+		 * can't change our nickname because we're banned.  The 
+		 * easiest way to tell is to see if we are on the channel.
+		 */
+		if (is_channel(ArgList[0]))
+		{
+		    if (!im_on_channel(ArgList[0], from_server))
+			remove_channel(ArgList[0], from_server);
+
+		    if (do_hook(current_numeric, "%s %s", from, ArgList[0]))
+			display_msg(from, ArgList);
+		    break;
+		}
+
+		/*
+		 * Otherwise, a nick command failed.  Oh boy.
+		 * If we are registered, abort the nick change and
+		 * hope for the best.
+		 */
+		else if (is_server_connected(from_server))
+		{
+			accept_server_nickname(from_server, user);
+			break;
+		}
+
+		/* 
+		 * Otherwise, it's an ircnet "nick not available" error.
+		 * Let the nickname reset numerics handle this mess.
+		 */
+		/* FALLTHROUGH */
+	}
+	case 433:		/* #define ERR_NICKNAMEINUSE    433 */ 
+	case 438:		/* EFnet/TS4 "nick collision" numeric 438 */
+	case 453:		/* EFnet/TS4 "nickname lost" numeric 453 */
+	{
+		if (get_int_var(AUTO_NEW_NICK_VAR))
+			fudge_nickname(from_server);
+		else
+			reset_nickname(from_server);
+
+		PasteArgs(ArgList, 0);
+		if (never_connected &&
+			do_hook(current_numeric, "-1 %s", *ArgList))
+				display_msg(from, ArgList);
+		else if (!never_connected && 
+			do_hook(current_numeric, "%s %s", from, *ArgList))
+				display_msg(from, ArgList);
+
+		break;
+	}
+
+	case 442:		/* #define ERR_NOTONCHANNEL	442 */
+	{
+		const char *	s;
+		if (!ArgList[0])
+			break;
+
+		PasteArgs(ArgList, 0);
+		s = get_server_name(from_server);
+		if (!my_strnicmp(s, from, strlen(s)))
+		{
+			if (strcmp(ArgList[0], "*"))
+			    remove_channel(ArgList[0], from_server);
+		}
+
+		/* Why wasn't this offered before? */
+		if (do_hook(current_numeric, "%s %s", from, *ArgList))
+			display_msg(from, ArgList);
+		break;
+	}
 
 	case 451:		/* #define ERR_NOTREGISTERED    451 */
 	/*
 	 * Sometimes the server doesn't catch the USER line, so
 	 * here we send a simplified version again  -lynx 
 	 */
-		send_to_server("USER %s %s . :%s", username,
-			(send_umode && *send_umode) ? send_umode : ".", realname);
-		send_to_server("NICK %s", get_server_nickname(from_server));
+		register_server(from_server, NULL);
 
 		PasteArgs(ArgList, 0);
 		if (do_hook(current_numeric, "%s %s", from, *ArgList))
@@ -651,7 +652,7 @@ numbered_command(from, comm, ArgList)
 
 	case 462:		/* #define ERR_ALREADYREGISTRED 462 */
 	{
-		send_to_server("NICK %s", get_server_nickname(from_server));
+		change_server_nickname(from_server, NULL);
 
 		PasteArgs(ArgList, 0);
 		if (do_hook(current_numeric, "%s %s", from, *ArgList))
@@ -665,34 +666,48 @@ numbered_command(from, comm, ArgList)
 		if (!is_server_connected(from_server))
 		{
 			say("Odd server stuff from %s: %s", from, ArgList[0]);
-			return;
+			break;
 		}
+
 		flag = do_hook(current_numeric, "%s %s", from, ArgList[0]);
 		if (oper_command)
 		{
 			if (flag)
 				display_msg(from, ArgList);
+			oper_command = 0;
 		}
 		else
-			get_password();
+		{
+			char	server_num[8];
+
+			say("Password required for connection to server %s",
+				get_server_name(from_server));
+			close_server(from_server, empty_string);
+			if (!dumb_mode)
+			{
+				strlcpy(server_num, ltoa(from_server), 8);
+				add_wait_prompt("Server Password:", 
+					password_sendline,
+				       server_num, WAIT_PROMPT_LINE, 0);
+			}
+		}
 		break;
 	}
 	case 465:		/* #define ERR_YOUREBANNEDCREEP 465 */
 	{
 		PasteArgs(ArgList, 0);
-		/* There used to be a say() here, but if we arent 
+		/* 
+		 * There used to be a say() here, but if we arent 
 		 * connected to a server, then doing say() is not
 		 * a good idea.  So now it just doesnt do anything.
 		 */
-		if (!is_server_connected(from_server))
-			return;
 		if (do_hook(current_numeric, "%s %s", from, ArgList[0]))
 			display_msg(from, ArgList);
 		close_server(from_server, empty_string);
 		window_check_servers();
 #ifndef I_DONT_TRUST_MY_USERS
 		if (from_server == primary_server)
-			get_connected(from_server + 1);
+			get_connected(from_server + 1, from_server);
 #endif
 		break;
 	}
@@ -704,12 +719,12 @@ numbered_command(from, comm, ArgList)
 	case 374:		/* #define RPL_ENDOFINFO        374 */
 	case 394:		/* #define RPL_ENDOFUSERS       394 */
 	{
-		if (get_int_var(SHOW_END_OF_MSGS_VAR))
-		{
-			PasteArgs(ArgList, 0);
-			if (do_hook(current_numeric, "%s %s", from, *ArgList))
-				display_msg(from, ArgList);
-		}
+		int hook;
+
+		PasteArgs(ArgList, 0);
+		hook = do_hook(current_numeric, "%s %s", from, *ArgList);
+		if (hook && get_int_var(SHOW_END_OF_MSGS_VAR))
+			display_msg(from, ArgList);
 		break;
 	}
 
@@ -745,7 +760,7 @@ numbered_command(from, comm, ArgList)
 			new_free(&ArgSpace);
 			if (do_message_from)
 				set_lastlog_msg_level(lastlog_level);
-			return;
+			break;
 		}
 		if (do_message_from)
 			message_from((char *) 0, lastlog_level);
@@ -765,61 +780,63 @@ numbered_command(from, comm, ArgList)
 				ArgList[0]);
 			break;
 		}
-		case 242:		/* #define RPL_STATSUPTIME      242 */
-		{
-			PasteArgs(ArgList, 0);
-			if (from && !my_strnicmp(get_server_itsname(from_server),
-			    from, strlen(get_server_itsname(from_server))))
-				from = NULL;
-			if (from)
-				put_it("%s %s from (%s)", numeric_banner(),
-					*ArgList, from);
-			else
-				put_it("%s %s", numeric_banner(), *ArgList);
-			break;
-		}
+
 		case 271:		/* #define SILENCE_LIST		271 */
 		{
 			put_it ("%s %s is ignoring %s", numeric_banner(), ArgList[0], ArgList[1]);
 			break;
 		}
-		case 272:		/* #define ENDOFSILENCE		272 */
-		{
-			PasteArgs(ArgList, 0);
-			put_it ("%s %s", numeric_banner(), ArgList[0]);
-			break;
-		}
+
 		case 329:		/* #define CREATION_TIME	329 */
 		{
-			time_t tme;
-			char *this_sucks;
+			/* Erf/TS4 support */
+			if (ArgList[3])
+			{
+				time_t tme1 = (time_t)my_atol(ArgList[1]);
+				time_t tme2 = (time_t)my_atol(ArgList[2]);
+				time_t tme3 = (time_t)my_atol(ArgList[3]);
 
-			sscanf(ArgList[1], "%lu", &tme);
-			this_sucks = ctime(&tme);
-			this_sucks[strlen(this_sucks)-1] = '\0';		
+				message_from(ArgList[0], LOG_CRAP);
+				put_it("%s Channel %s was created at %ld, +c was last set at %ld, and has been opless since %ld", numeric_banner(), ArgList[0], tme1, tme2, tme3);
+				message_from((char *) 0, LOG_CURRENT);
+			}
+			if (ArgList[1])
+			{
+				time_t tme = (time_t)my_atol(ArgList[1]);
 
-			message_from(ArgList[0], LOG_CRAP);
-			put_it("%s Channel %s was created at %s",numeric_banner(),
-				ArgList[0], this_sucks);
-			message_from((char *) 0, LOG_CURRENT);
+				message_from(ArgList[0], LOG_CRAP);
+				put_it("%s Channel %s was created at %s",
+						numeric_banner(),
+					ArgList[0], my_ctime(tme));
+				message_from((char *) 0, LOG_CURRENT);
+			}
 			break;
 		}
 		case 332:		/* #define RPL_TOPIC            332 */
-			channel_topic(from, ArgList);
+		{
+			message_from(ArgList[0], LOG_CRAP);
+			put_it("%s Topic for %s: %s", numeric_banner(), ArgList[0], ArgList[1]);
 			break;
+		}
 
 		case 333:		/* #define RPL_TOPICWHOTIME	333 */
 		{
-			unsigned long tme;
-			/* Ooops. Texaco caught this bug */
-			tme = (unsigned long)atol(ArgList[2]);
-			put_it("%s The topic was set by %s %lu sec ago",numeric_banner(), 
-				ArgList[1], (unsigned long)(time(NULL)-tme));
+			/* Bug in aircd makes this check neccesary.  */
+			if (ArgList[2])
+			{
+				time_t tme = (unsigned long)my_atol(ArgList[2]);
+				put_it("%s The topic was set by %s %lu sec ago",numeric_banner(), 
+					ArgList[1], (unsigned long)(time(NULL)-tme));
+			}
 			break;
 		}
 		case 351:		/* #define RPL_VERSION          351 */
-			version(from, ArgList);
+		{
+			PasteArgs(ArgList, 2);
+			put_it("%s Server %s: %s %s", numeric_banner(), ArgList[1],
+				ArgList[0], ArgList[2]);
 			break;
+		}
 
 		case 364:		/* #define RPL_LINKS            364 */
 		{
@@ -839,127 +856,85 @@ numbered_command(from, comm, ArgList)
 		}
 
 		case 384:		/* #define RPL_MYPORTIS         384 */
+		{
 			PasteArgs(ArgList, 0);
 			put_it("%s %s %s", numeric_banner(), ArgList[0], user);
 			break;
+		}
 
 		case 403:		/* #define ERR_NOSUCHCHANNEL    403 */
-			not_valid_channel(from, ArgList);
+		{
+			const char	*s;
+
+			if (!ArgList[1])
+				break;
+
+			PasteArgs(ArgList, 1);
+			s = get_server_name(from_server);
+			if (!my_strnicmp(s, from, strlen(s)))
+			{
+				/* hack cause channel may be null */
+				if (ArgList[0])
+				{
+					/*
+					 * Dont blindly remove channel '*' (cb)
+					 */
+					if (strcmp(ArgList[0], "*"))
+						remove_channel(ArgList[0], from_server);
+					put_it("%s %s %s", numeric_banner(), ArgList[0], ArgList[1]);
+				}
+				else
+					put_it("%s You did not specify a channel", numeric_banner());
+			}
 			break;
+		}
 
 		case 471:		/* #define ERR_CHANNELISFULL    471 */
 		case 473:		/* #define ERR_INVITEONLYCHAN   473 */
 		case 474:		/* #define ERR_BANNEDFROMCHAN   474 */
 		case 475: 		/* #define ERR_BADCHANNELKEY    475 */
 		case 476:		/* #define ERR_BADCHANMASK      476 */
-			cannot_join_channel(from, ArgList);
+		{
+			char buffer[BIG_BUFFER_SIZE + 1];
+
+			if (ArgList[0])
+				remove_channel(ArgList[0], from_server);
+			PasteArgs(ArgList, 0);
+			strmcpy(buffer, ArgList[0], BIG_BUFFER_SIZE);
+			switch(comm)
+			{
+				case 471:
+					strmcat(buffer, " (Channel is full)", BIG_BUFFER_SIZE);
+					break;
+				case 473:
+					strmcat(buffer, " (You must be invited)", BIG_BUFFER_SIZE);
+					break;
+				case 474:
+					strmcat(buffer, " (You are banned)", BIG_BUFFER_SIZE);
+					break;
+				case 475:
+					strmcat(buffer, " (You must give the correct key)", BIG_BUFFER_SIZE);
+					break;
+				case 476:
+					strmcat(buffer, " (Bad channel mask)", BIG_BUFFER_SIZE);
+					break;
+			}
+			put_it("%s %s", numeric_banner(), buffer);
 			break;
+		}
 
 #define RPL_CLOSEEND         363
-#define RPL_SERVLISTEND      235
 		case 323:               /* #define RPL_LISTEND          323 */
 			funny_print_widelist();
+			/* FALLTHROUGH */
 
 		default:
 			display_msg(from, ArgList);
 		}
 	}
+
+	current_numeric = old_current_numeric;
 	set_lastlog_msg_level(lastlog_level);
 }
 
 
-/* Dont even think of calling this function with anything that isnt
- * really a nickname -- it may cause segfaults!
- */
-/* This will generate up to 18 nicknames plus the 9-length(nickname)
- * that are unique but still have some semblance of the original.
- * This is intended to allow the user to get signed back on to
- * irc after a nick collision without their having to manually
- * type a new nick every time..
- * 
- * The func will try to make an intelligent guess as to when it is
- * out of guesses, and if it ever gets to that point, it will do the
- * manually-ask-you-for-a-new-nickname thing.
- */
-static void fudge_nickname _((void))
-{
-	static char oldnick[LONG_NICKNAME_LEN+1] = {0};
-	static int fudge_index = 0;
-	char blah[4];
-	extern int from_server;
-	char nickname[LONG_NICKNAME_LEN + 1] = {0};
-
-	if (in_e_nick)
-		return;		/* if we're changing nickname, dont bother */
-
-	strcpy(nickname, get_server_nickname(from_server));
-
-	if (!get_int_var(AUTO_NEW_NICK_VAR))
-	{
-		reset_nickname();
-		return;
-	}
-
-	/* First time through... */
-	if (!*oldnick)
-	{
-		strncpy(oldnick, nickname,
-#ifdef ALLOW_LONG_NICKNAMES
-						LONG_NICKNAME_LEN
-#else
-						NICKNAME_LEN
-#endif
-									);
-		fudge_index = strlen(nickname);
-	}
-	else
-	{
-		/* Same nick as the last one we convoluted? */
-		if (!my_stricmp(oldnick, nickname))
-		{
-			fudge_index++;
-			if (fudge_index == 17)
-			{
-				/* give up... */
-				reset_nickname();
-				fudge_index = 0;
-				return;
-			}
-		}
-		/* Different nick then last time.. start over. */
-		else
-		{
-			strcpy(oldnick, nickname);
-			fudge_index = strlen(nickname);
-		}
-	}
-
-	/* 
-	 * Process of fudging a nickname:
-	 * If the nickname length is less then 9, add an underscore.
-	 */
-	if (strlen(nickname) < 9)
-		strcat(nickname, "_");
-
-	/* 
-	 * The nickname is 9 characters long. roll the nickname
-	 */
-	else
-	{
-		char tmp = nickname[8];
-		nickname[8] = nickname[7];
-		nickname[7] = nickname[6];
-		nickname[6] = nickname[5];
-		nickname[5] = nickname[4];
-		nickname[4] = nickname[3];
-		nickname[3] = nickname[2];
-		nickname[2] = nickname[1];
-		nickname[1] = nickname[0];
-		nickname[0] = tmp;
-	}
-
-	strcpy(blah, ltoa(from_server));
-	strcpy(oldnick, nickname);
-	nickname_sendline(blah, nickname);
-}
-
