@@ -40,6 +40,7 @@
 const 	char *	umodes = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
 static	char *	do_umode (int du_index);
 	void 	reset_nickname (int);
+static void	clear_reconnect_counts (void);
 
 	int	never_connected = 1;		/* true until first connection
 						 * is made */
@@ -449,20 +450,22 @@ int 	read_server_file (void)
 #endif
 	}
 
-	expanded = expand_twiddle(file_path);
-	if ((fp = fopen(expanded, "r")))
+	if ((expanded = expand_twiddle(file_path)))
 	{
-		while (fgets(buffer, BIG_BUFFER_SIZE, fp))
+		if ((fp = fopen(expanded, "r")))
 		{
-			chop(buffer, 1);
-			build_server_list(buffer);
+			while (fgets(buffer, BIG_BUFFER_SIZE, fp))
+			{
+				chop(buffer, 1);
+				build_server_list(buffer);
+			}
+			fclose(fp);
+			new_free(&expanded);
+			return (0);
 		}
-		fclose(fp);
-		new_free(&expanded);
-		return (0);
-	}
 
-	new_free(&expanded);
+		new_free(&expanded);
+	}
 	return 1;
 }
 
@@ -608,6 +611,8 @@ BUILT_IN_COMMAND(servercmd)
 	 */
 	else if (*server == '+')
 	{
+		clear_reconnect_counts();
+
 		/* /SERVER +foo.bar.com is an alias for /window server */
 		if (*++server)
 			window_server(current_window, &server);
@@ -624,6 +629,8 @@ BUILT_IN_COMMAND(servercmd)
 	 */
 	else if (*server == '-')
 	{
+		clear_reconnect_counts();
+
 		if (*++server)
 		{
 			i = find_server_refnum(server, &args);
@@ -648,6 +655,7 @@ BUILT_IN_COMMAND(servercmd)
 	{
 		int j = from_server;
 
+		clear_reconnect_counts();
 		i = find_server_refnum(server, &args);
 		connect_to_new_server(i, j, 0);
 		window_check_servers();
@@ -896,9 +904,10 @@ static int 	connect_to_server (int new_server)
 	/*
 	 * Can't connect to refnum -1, this is definitely an error.
 	 */
-	if (new_server == -1)
+	if (new_server < 0)
 	{
-		say ("Connecting to refnum -1.  That makes no sense.");
+		say("Connecting to refnum %d.  That makes no sense.", 
+			new_server);
 		return -1;		/* XXXX */
 	}
 
@@ -1122,12 +1131,23 @@ int 	connect_to_new_server (int new_server, int old_server, int new_conn)
  * tries as many servers as it takes to get a new server connection going.
  * If all else fails, it reverts back to the original connection, provided
  * that the original connection existed. ;-)
+ *
+ * This is the lynchpin through which all of the auto-reconnect madness 
+ * happens.  Because it is recursive, bad things can happen if a server
+ * connection behaves in a way that we are not prepared to handle.  Therefore
+ * the impetus is on this function to refuse to act and disconnect the user
+ * and force manual intervention if it appears we are spinning out of control.
+ * Because recursive autoreconnects can occur, we use the "reconnects" member
+ * of the server structure to throttle unsuccessfull reconnection attempts 
+ * here.  The number of unsuccessfull (defined as not completing registration)
+ * attempts is controlled by /set max_reconnect_attempts.
  */
 int 	get_connected (int new, int old)
 {
 	int	i;
 	int	j;
 	int	connected;
+	int	max_reconnects = get_int_var(MAX_RECONNECTS_VAR);
 
 	if (!server_list)
 	{
@@ -1136,6 +1156,9 @@ int 	get_connected (int new, int old)
 		return -1;
 	}
 
+	if (new < 0)
+		new = number_of_servers - 1;		/* Sanity check */
+
 	connected = is_server_connected(old);
 
 	/* Try all of the other servers, stop when one of them works. */
@@ -1144,6 +1167,10 @@ int 	get_connected (int new, int old)
 		j = (i + new) % number_of_servers;
 		if (new != old && j == old)
 			continue;
+		if ((server_list[j].reconnects++) > max_reconnects) {
+			say("Auto-reconnect has been throttled because too many unsuccessfull attempts to connect to server %d have been performed.", j);
+			break;
+		}
 		if (connect_to_new_server(j, old, 0) == 0)
 			return j;
 	}
@@ -1195,6 +1222,8 @@ int 	close_all_servers (const char *message)
  */
 void	close_server (int old, const char *message)
 {
+	int	was_connected;
+
 	/* Make sure server refnum is valid */
 	if (old < 0 || old >= number_of_servers)
 	{
@@ -1205,6 +1234,8 @@ void	close_server (int old, const char *message)
 	if (server_list[old].des == -1)
 		return;			/* Can't close server that isn't open */
 
+	was_connected = server_list[old].connected;
+
 	clean_server_queues(old);
 	if (waiting_out > waiting_in)		/* XXX - hack! */
 		waiting_out = waiting_in = 0;
@@ -1230,7 +1261,13 @@ void	close_server (int old, const char *message)
 			yell("Closing server %d because [%s]", 
 			   old, message ? message : empty_string);
 
-		    send_to_aserver(old, "QUIT :%s\n", message);
+		    /*
+		     * Only tell the server we are leaving if we are 
+		     * registered.  This avoids an infinite loop in the
+		     * D-line case.
+		     */
+		    if (was_connected)
+			    send_to_aserver(old, "QUIT :%s\n", message);
 		    do_hook(SERVER_LOST_LIST, "%d %s %s", 
 				old, server_list[old].name, message);
 		}
@@ -1565,7 +1602,10 @@ void 	server_is_connected (int sic_index, int value)
 	server_list[sic_index].connected = value;
 	server_list[sic_index].registration_pending = 0;
 	if (value)
+	{
 		server_list[sic_index].eof = 0;
+		clear_reconnect_counts();
+	}
 }
 
 BUILT_IN_COMMAND(disconnectcmd)
@@ -1998,3 +2038,11 @@ void 	save_servers (FILE *fp)
 	}
 }
 
+static void	clear_reconnect_counts (void)
+{
+	int	i;
+
+	for (i = 0; i < number_of_servers; i++)
+		server_list[i].reconnects = 0;
+}
+
